==> ipf_dynamic_attributes-1.0.0-py3-none-any.whl: ipf_dynamic_attributes/configs/rules.py <==
from functools import cached_property
from typing import Optional, Union, ForwardRef

from pydantic import BaseModel, Field, model_validator

from ipf_dynamic_attributes.configs.filters import Filter, AttributeFilter, DeviceFilter
from ipf_dynamic_attributes.configs.values import AttributeValue, ConfigValue

REPORT_COLS = [
    "attribute",
    "global_value",
    "local_value",
    "no_config",
    "name",
    "api_endpoint",
    "no_start_config",
    "no_current_config",
    "new_value",
    "column_value",
    "static",
    "null_value",
    "regex_pattern",
    "regex_group",
    "regex_match",
    "mapping_match",
    "delete",
]
DELETE_ATTRIBUTE = Field(
    None,
    description="Caution: If no match is found this will delete the Attribute from all devices in the filtered "
    "inventory list for this Attribute Name. After running each rule and no match is found, the "
    "attribute will be flagged for deletion in the remaining inventory list. If a later rule matches "
    "does find a match then the attribute will not be deleted and the new value will be assigned.",
    title="Delete Attribute (DANGEROUS - Read the Description)",
)
RULE_NAME = Field(
    description="A unique name of the rule for reporting purposes.",
    title="Rule Name",
    examples=["Rule 1", "Loopback0 Rule"],
)
ATTRIBUTE = Field(
    None,
    description="The name of the Attribute to be updated or created.",
    title="Attribute Name",
    examples=["COUNTRY", "MGMT_IP", "ENVIRONMENT"],
)

Rule, ConfigRule = ForwardRef("Rule"), ForwardRef("ConfigRule")
DefaultRule, DefaultConfigRule = ForwardRef("DefaultRule"), ForwardRef("DefaultConfigRule")


def merge_default(
    original: Union[Rule, ConfigRule], default: Union[DefaultRule, DefaultConfigRule]
) -> Union[Rule, ConfigRule]:
    if not default:
        return original
    # Copy Default Attribute Name to Rule:
    original.attribute = original.attribute or default.attribute
    if default.delete_attribute is not None and original.delete_attribute is None:
        # Copy Default Delete Attribute to Rule if Rule does not have delete_attribute set:
        original.delete_attribute = default.delete_attribute
    if original.merge_default_filters:
        if default.filter_string and not original.filters:
            # Copy Default Filter String to Rule if Rule does not have filters and merge_default_filters is True:
            original.filter_string = default.filter_string
        if default.filters and not original.filter_string:
            # Copy Default Filters to Rule if Rule does not have a filter_string and merge_default_filters is True:
            original.filters.extend(default.filters)
    return original


class FilterRule(BaseModel):
    filters: list[Filter] = Field(
        default_factory=list, description="List of filters to apply to table endpoint.", title="Table Filters"
    )
    filter_string: Optional[str] = Field(
        None,
        description="A string to use for the filter on table endpoint instead of the filters list.",
        title="Table Filter String",
        examples=['{"vendor":["like","cisco"]}'],
    )

    @model_validator(mode="after")
    def _validate(self):
        if self.filters and self.filter_string:
            raise ValueError("'rule[filter_string]' cannot be combined with 'rule[filters]'.")
        return self

    @property
    def filter_list(self) -> list[dict]:
        return [_.format for _ in self.filters]

    @property
    def calc_filter(self) -> Union[dict[str, list[dict]], str]:
        if self.filter_string:
            return self.filter_string
        return {"and": self.filter_list} if self.filter_list else dict()


class BaseRule(FilterRule, BaseModel):
    attribute_filters: list[AttributeFilter] = Field(
        default_factory=list, description="Attribute Filters to apply to table endpoint.", title="Attribute Filters"
    )

    @model_validator(mode="after")
    def _validate(self):
        if (self.filters or self.attribute_filters) and self.filter_string:
            raise ValueError(
                "'rule[filter_string]' cannot be combined with 'rule[filters]' or 'rule[attribute_filters]'."
            )
        return self

    @property
    def filter_list(self) -> list[dict]:
        return [*super().filter_list, *[_.format for _ in self.attribute_filters]]


class InventoryRule(BaseRule, BaseModel):
    report_columns: list[str] = Field(
        default_factory=list,
        description="List of Inventory > Devices columns to also include in the Sync Report; "
        "the report will always return [hostname, sn, siteName].",
        title="Inventory Report Columns",
        examples=["model"],
    )

    @cached_property
    def calc_columns(self) -> list[str]:
        """Return the columns to use for the Inventory."""
        return list({"hostname", "sn", "siteName", *self.report_columns})

    @cached_property
    def df_columns(self) -> list[str]:
        """Return the columns to use for the DataFrame."""
        return [*self.calc_columns, *REPORT_COLS]


class DefaultRule(BaseRule, BaseModel):
    attribute: Optional[str] = ATTRIBUTE
    value: Optional[AttributeValue] = Field(
        None,
        description="The value to assign to the Attribute, "
        "either configured in the Default Table rules or per Table rule.",
        title="Attribute Value",
    )
    device_filters: list[DeviceFilter] = Field(
        default_factory=list,
        description="Device filter to apply to the table endpoint (i.e. vendor, family, etc.).",
        title="Device Filters",
    )
    delete_attribute: Optional[bool] = DELETE_ATTRIBUTE

    @model_validator(mode="after")
    def _validate(self):
        if (self.filters or self.attribute_filters or self.device_filters) and self.filter_string:
            raise ValueError(
                "'rule[filter_string]' cannot be combined with 'rule[filters]', "
                "'rule[attribute_filters]', or 'rule[device_filters]'."
            )
        return self

    @property
    def filter_list(self) -> list[dict]:
        return [*super().filter_list, *[_.format for _ in self.device_filters]]


class Rule(DefaultRule, BaseModel):
    name: str = RULE_NAME
    merge_default_filters: bool = Field(
        True,
        description="Merge 'default[filters]' with Table Rule filters or "
        "'default[filter_string]' to Table Rule filter_string.",
        title="Merge Default Filters",
    )
    merge_default_device_filters: bool = Field(
        True,
        description="Merge 'default[device_filters]' with Table Rule device_filters.",
        title="Merge Default Device Filters",
    )
    merge_default_attribute_filters: bool = Field(
        True,
        description="Merge 'default[attribute_filters]' with Table Rule attribute_filters.",
        title="Merge Default Attribute Filters",
    )

    def merge_default(self, default: DefaultRule) -> Rule:
        if not isinstance(default, DefaultRule):
            return self
        rule = merge_default(self.model_copy(deep=True), default)
        # Copy Default Value Config to Rule:
        rule.value = rule.value or default.value
        # Copy Default SN Column to Rule:
        rule.value.sn_column = rule.value.sn_column or (default.value.sn_column if default.value else "sn")
        if default.delete_attribute is not None and rule.delete_attribute is None:
            # Copy Default Delete Attribute to Rule if Rule does not have delete_attribute set:
            rule.delete_attribute = default.delete_attribute
        if not rule.filter_string:
            if default.device_filters and rule.merge_default_device_filters:
                # Copy Default Device Filters to Rule if merge_default_filters is True and filter_string is not set:
                rule.device_filters.extend(default.device_filters)
            if default.attribute_filters and rule.merge_default_attribute_filters:
                # Copy Default Attribute Filters to Rule if merge_default_attribute_filters is True:
                rule.attribute_filters.extend(default.attribute_filters)
        return rule


class DefaultConfigRule(FilterRule, BaseModel):
    attribute: Optional[str] = ATTRIBUTE
    delete_attribute: Optional[bool] = DELETE_ATTRIBUTE
    filters: list[Filter] = Field(
        default_factory=list,
        description="Additional Inventory Filters to apply to limit devices for configuration matching.",
        title="Additional Inventory Filters",
    )
    filter_string: Optional[str] = Field(
        None,
        description="A string to use for additional Inventory Filtering instead of the filters list.",
        title="Additional Inventory Filter String",
        examples=['{"vendor":["like","cisco"]}'],
    )

    @model_validator(mode="after")
    def _validate(self):
        if self.filters and self.filter_string:
            raise ValueError("'rule[filter_string]' cannot be combined with 'rule[filters]'.")
        return self


class ConfigRule(DefaultConfigRule, BaseModel):
    name: str = RULE_NAME
    value: ConfigValue = Field(
        description="The value to assign to the Attribute based on a configuration search.",
        title="Attribute Value",
    )
    merge_default_filters: bool = Field(
        True,
        description="Merge 'default[config_filters]' with Configuration Rule filters.",
        title="Merge Default Configuration Filters",
    )

    def merge_default(self, default: DefaultConfigRule) -> ConfigRule:
        """Merge Default Config Rule with Config Rule"""
        return merge_default(self.model_copy(deep=True), default) if isinstance(default, DefaultConfigRule) else self


Rule.model_rebuild()
ConfigRule.model_rebuild()
DefaultRule.model_rebuild()
DefaultConfigRule.model_rebuild()

==> ipf_dynamic_attributes-1.0.1-py3-none-any.whl: ipf_dynamic_attributes/configs/rules.py <==
from functools import cached_property
from typing import Optional, Union, ForwardRef

from pydantic import BaseModel, Field, model_validator

from ipf_dynamic_attributes.configs.filters import Filter, AttributeFilter, DeviceFilter
from ipf_dynamic_attributes.configs.values import TableValue, ConfigValue

REPORT_COLS = [
    "attribute",
    "global_value",
    "local_value",
    "no_config",
    "name",
    "api_endpoint",
    "no_start_config",
    "no_current_config",
    "new_value",
    "column_value",
    "static",
    "null_value",
    "regex_pattern",
    "regex_group",
    "regex_match",
    "mapping_match",
    "delete",
]
DELETE_ATTRIBUTE = Field(
    None,
    description="Caution: If no match is found this will delete the Attribute from all devices in the filtered "
    "inventory list for this Attribute Name. After running each rule and no match is found, the "
    "attribute will be flagged for deletion in the remaining inventory list. If a later rule matches "
    "does find a match then the attribute will not be deleted and the new value will be assigned.",
    title="Delete Attribute (DANGEROUS - Read the Description)",
)
RULE_NAME = Field(
    description="A unique name of the rule for reporting purposes.",
    title="Rule Name",
    examples=["Rule 1", "Loopback0 Rule"],
)
ATTRIBUTE = Field(
    None,
    description="The name of the Attribute to be updated or created.",
    title="Attribute Name",
    examples=["COUNTRY", "MGMT_IP", "ENVIRONMENT"],
)

Rule, ConfigRule = ForwardRef("Rule"), ForwardRef("ConfigRule")
DefaultRule, DefaultConfigRule = ForwardRef("DefaultRule"), ForwardRef("DefaultConfigRule")


def merge_default(
    original: Union[Rule, ConfigRule], default: Union[DefaultRule, DefaultConfigRule]
) -> Union[Rule, ConfigRule]:
    if not default:
        return original
    # Copy Default Attribute Name to Rule:
    original.attribute = original.attribute or default.attribute
    if default.delete_attribute is not None and original.delete_attribute is None:
        # Copy Default Delete Attribute to Rule if Rule does not have delete_attribute set:
        original.delete_attribute = default.delete_attribute
    if original.merge_default_filters:
        if default.filter_string and not original.filters:
            # Copy Default Filter String to Rule if Rule does not have filters and merge_default_filters is True:
            original.filter_string = default.filter_string
        if default.filters and not original.filter_string:
            # Copy Default Filters to Rule if Rule does not have a filter_string and merge_default_filters is True:
            original.filters.extend(default.filters)
    return original


class FilterRule(BaseModel):
    filters: list[Filter] = Field(
        default_factory=list, description="List of filters to apply to table endpoint.", title="Table Filters"
    )
    filter_string: Optional[str] = Field(
        None,
        description="A string to use for the filter on table endpoint instead of the filters list.",
        title="Table Filter String",
        examples=['{"vendor":["like","cisco"]}'],
    )

    @model_validator(mode="after")
    def _validate(self):
        if self.filters and self.filter_string:
            raise ValueError("'rule[filter_string]' cannot be combined with 'rule[filters]'.")
        return self

    @property
    def filter_list(self) -> list[dict]:
        return [_.format for _ in self.filters]

    @property
    def calc_filter(self) -> Union[dict[str, list[dict]], str]:
        if self.filter_string:
            return self.filter_string
        return {"and": self.filter_list} if self.filter_list else dict()


class BaseRule(FilterRule, BaseModel):
    attribute_filters: list[AttributeFilter] = Field(
        default_factory=list, description="Attribute Filters to apply to table endpoint.", title="Attribute Filters"
    )

    @model_validator(mode="after")
    def _validate(self):
        if (self.filters or self.attribute_filters) and self.filter_string:
            raise ValueError(
                "'rule[filter_string]' cannot be combined with 'rule[filters]' or 'rule[attribute_filters]'."
            )
        return self

    @property
    def filter_list(self) -> list[dict]:
        return [*super().filter_list, *[_.format for _ in self.attribute_filters]]


class InventoryRule(BaseRule, BaseModel):
    report_columns: list[str] = Field(
        default_factory=list,
        description="List of Inventory > Devices columns to also include in the Sync Report; "
        "the report will always return [hostname, sn, siteName].",
        title="Inventory Report Columns",
        examples=["model"],
    )

    @cached_property
    def calc_columns(self) -> list[str]:
        """Return the columns to use for the Inventory."""
        return list({"hostname", "sn", "siteName", *self.report_columns})

    @cached_property
    def df_columns(self) -> list[str]:
        """Return the columns to use for the DataFrame."""
        return [*self.calc_columns, *REPORT_COLS]


class DefaultRule(BaseRule, BaseModel):
    attribute: Optional[str] = ATTRIBUTE
    value: Optional[TableValue] = Field(
        None,
        description="The value to assign to the Attribute, "
        "either configured in the Default Table rules or per Table rule.",
        title="Attribute Value",
    )
    device_filters: list[DeviceFilter] = Field(
        default_factory=list,
        description="Device filter to apply to the table endpoint (i.e. vendor, family, etc.).",
        title="Device Filters",
    )
    delete_attribute: Optional[bool] = DELETE_ATTRIBUTE

    @model_validator(mode="after")
    def _validate(self):
        if (self.filters or self.attribute_filters or self.device_filters) and self.filter_string:
            raise ValueError(
                "'rule[filter_string]' cannot be combined with 'rule[filters]', "
                "'rule[attribute_filters]', or 'rule[device_filters]'."
            )
        return self

    @property
    def filter_list(self) -> list[dict]:
        return [*super().filter_list, *[_.format for _ in self.device_filters]]


class Rule(DefaultRule, BaseModel):
    name: str = RULE_NAME
    merge_default_filters: bool = Field(
        True,
        description="Merge 'default[filters]' with Table Rule filters or "
        "'default[filter_string]' to Table Rule filter_string.",
        title="Merge Default Filters",
    )
    merge_default_device_filters: bool = Field(
        True,
        description="Merge 'default[device_filters]' with Table Rule device_filters.",
        title="Merge Default Device Filters",
    )
    merge_default_attribute_filters: bool = Field(
        True,
        description="Merge 'default[attribute_filters]' with Table Rule attribute_filters.",
        title="Merge Default Attribute Filters",
    )

    def merge_default(self, default: DefaultRule) -> Rule:
        if not isinstance(default, DefaultRule):
            return self
        rule = merge_default(self.model_copy(deep=True), default)
        # Copy Default Value Config to Rule:
        rule.value = rule.value or default.value
        # Copy Default SN Column to Rule:
        rule.value.sn_column = rule.value.sn_column or (default.value.sn_column if default.value else "sn")
        if default.delete_attribute is not None and rule.delete_attribute is None:
            # Copy Default Delete Attribute to Rule if Rule does not have delete_attribute set:
            rule.delete_attribute = default.delete_attribute
        if not rule.filter_string:
            if default.device_filters and rule.merge_default_device_filters:
                # Copy Default Device Filters to Rule if merge_default_filters is True and filter_string is not set:
                rule.device_filters.extend(default.device_filters)
            if default.attribute_filters and rule.merge_default_attribute_filters:
                # Copy Default Attribute Filters to Rule if merge_default_attribute_filters is True:
                rule.attribute_filters.extend(default.attribute_filters)
        return rule


class DefaultConfigRule(FilterRule, BaseModel):
    attribute: Optional[str] = ATTRIBUTE
    delete_attribute: Optional[bool] = DELETE_ATTRIBUTE
    filters: list[Filter] = Field(
        default_factory=list,
        description="Additional Inventory Filters to apply to limit devices for configuration matching.",
        title="Additional Inventory Filters",
    )
    filter_string: Optional[str] = Field(
        None,
        description="A string to use for additional Inventory Filtering instead of the filters list.",
        title="Additional Inventory Filter String",
        examples=['{"vendor":["like","cisco"]}'],
    )

    @model_validator(mode="after")
    def _validate(self):
        if self.filters and self.filter_string:
            raise ValueError("'rule[filter_string]' cannot be combined with 'rule[filters]'.")
        return self


class ConfigRule(DefaultConfigRule, BaseModel):
    name: str = RULE_NAME
    value: ConfigValue = Field(
        description="The value to assign to the Attribute based on a configuration search.",
        title="Attribute Value",
    )
    merge_default_filters: bool = Field(
        True,
        description="Merge 'default[config_filters]' with Configuration Rule filters.",
        title="Merge Default Configuration Filters",
    )

    def merge_default(self, default: DefaultConfigRule) -> ConfigRule:
        """Merge Default Config Rule with Config Rule"""
        return merge_default(self.model_copy(deep=True), default) if isinstance(default, DefaultConfigRule) else self


Rule.model_rebuild()
ConfigRule.model_rebuild()
DefaultRule.model_rebuild()
DefaultConfigRule.model_rebuild()
